
//============================================================================

// Copyright (c) IFM Infomaster. All rights reserved.

//============================================================================

(function() {
  'use strict';

  window.Ifm = window.Ifm || {};

  Ifm.pluginLoader = {

    baseUrl: '',

    loadUrl: '',

    language: '',

    load: function(baseUrl, language) {
      self.baseUrl = baseUrl;
      self.language = language;

      loadConfiguration();
    },

    loadPlugin: function(url, deps) {
      if (!config) {
        console.warn('[PluginLoader] not loaded');
        return;
      }

      loadScripts(url, deps.scripts);
      loadStylesheets(url, deps.stylesheets);
    }

  };

  var self = Ifm.pluginLoader;
  var loader = Ifm.loader;

  var config;

  function isReady() {
    var state = document.readyState;
    return state === 'interactive' || state === 'complete';
  }

  function onReadyStateChange() {
    if (isReady()) {
      document.removeEventListener('readystatechange', onReadyStateChange);
      loadPlugins();
    }
  }

  function loadConfiguration() {
    var configUrl = self.baseUrl + '/pluginloader.config.json';
    loader.loadJSON(configUrl).then(function(configData) {
      config = configData;

      if (!config.enabled) {
        console.info('[PluginLoader] disabled by configuration');
        return;
      }

      self.loadUrl = self.baseUrl + '/' + config.baseUrl;

      if (isReady()) {
        loadPlugins();
      } else {
        document.addEventListener('readystatechange', onReadyStateChange);
      }
    }, function() {
      console.info('[PluginLoader] missing configuration');
    });
  }

  function loadPlugins() {
    config.plugins.forEach(function(pluginName) {
      var pluginBaseUrl = self.loadUrl + '/' + pluginName;
      var pluginDepsUrl = pluginBaseUrl + '/' + pluginName + '.deps.json';

      loader.loadJSON(pluginDepsUrl).then(function(pluginDeps) {
        self.loadPlugin(pluginBaseUrl, pluginDeps);
      });
    });
  }

  function loadScripts(baseUrl, scripts) {
    var script = scripts.shift();
    if (script) {
      var url = baseUrl + '/' + script.replace('{LANG}', self.language);
      loader.loadScript(url).then(function() {
        loadScripts(baseUrl, scripts);
      });
    }
  }

  function loadStylesheets(baseUrl, stylesheets) {
    var stylesheet = stylesheets.shift();
    if (stylesheet) {
      var url = baseUrl + '/' + stylesheet.replace('{LANG}', self.language);
      loader.loadStylesheet(url).then(function() {
        loadStylesheets(baseUrl, stylesheets);
      });
    }
  }

})();
